# Copyright (c) 2015 Oracle and/or its affiliates. All rights reserved. This
# code is released under a tri EPL/GPL/LGPL license. You can use it,
# redistribute it and/or modify it under the terms of the:
#
# Eclipse Public License version 1.0
# GNU General Public License version 2
# GNU Lesser General Public License version 2.1

# Copyright (c) 2007-2015, Evan Phoenix and contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of Rubinius nor the names of its contributors
#   may be used to endorse or promote products derived from this software
#   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

module Rubinius
  class Mirror
    def self.subject=(klass)
      @subject = klass
    end

    def self.module_mirror(obj)
      case obj
        when ::Numeric then Rubinius::Mirror::Numeric
        when ::String then Rubinius::Mirror::String
        when ::Range then Rubinius::Mirror::Range
        when ::Process then Rubinius::Mirror::Process
        else
          begin
            Rubinius::Mirror.const_get(obj.class.name.to_sym, false)
          rescue NameError
            ancestor = obj.class.superclass

            until ancestor.nil?
              begin
                return Rubinius::Mirror.const_get(ancestor.name.to_sym, false)
              rescue NameError
                ancestor = ancestor.superclass
              end
            end

            nil
          end
      end
    end

    def self.subject
      @subject
    end

    def self.reflect(obj)
      klass = module_mirror(obj)
      klass.new obj if klass
    end

    attr_reader :object

    def initialize(obj)
      @object = obj
    end

    class Object < Mirror
      self.subject = ::Object
    end

    def inspect
      "#<#{self.class.name}:0x#{self.object_id.to_s(16)} object=#{@object.inspect}>"
    end
  end
end
